<?php
// $Id: aggregator.admin.inc,v 1.7.2.1 2008/04/25 21:11:57 goba Exp $

/**
 * @file
 * Admin page callbacks for the aggregator module.
 */

/**
 * Menu callback; displays the aggregator administration page.
 */
function aggregator_admin_overview() {
  return aggregator_view();
}

/**
 * Displays the aggregator administration page.
 *
 * @return
 *   The page HTML.
 */
function aggregator_view() {
  //{m2osw
  $result = db_query('SELECT f.*, COUNT(i.iid) AS items FROM {aggregator_feed} f LEFT JOIN {aggregator_item} i ON f.fid = i.fid GROUP BY f.fid, f.title, f.url, f.refresh, f.checked, f.link, f.description, f.etag, f.modified, f.image, f.block, f.show_block, f.node_type ORDER BY f.title');
  //m2osw}

  $output = '<h3>'. t('Feed overview') .'</h3>';

  $header = array(t('Title'), t('Items'), t('Last update'), t('Next update'), array('data' => t('Operations'), 'colspan' => '3'));
  $rows = array();
  while ($feed = db_fetch_object($result)) {
    $rows[] = array(l($feed->title, "aggregator/sources/$feed->fid"), format_plural($feed->items, '1 item', '@count items'), ($feed->checked ? t('@time ago', array('@time' => format_interval(time() - $feed->checked))) : t('never')), ($feed->checked ? t('%time left', array('%time' => format_interval($feed->checked + $feed->refresh - time()))) : t('never')), l(t('edit'), "admin/content/aggregator/edit/feed/$feed->fid"), l(t('remove items'), "admin/content/aggregator/remove/$feed->fid"), l(t('update items'), "admin/content/aggregator/update/$feed->fid"));
  }
  $output .= theme('table', $header, $rows);

  $result = db_query('SELECT c.cid, c.title, count(ci.iid) as items FROM {aggregator_category} c LEFT JOIN {aggregator_category_item} ci ON c.cid = ci.cid GROUP BY c.cid, c.title ORDER BY title');

  $output .= '<h3>'. t('Category overview') .'</h3>';

  //{m2osw
  $header = array(t('Id'), t('Title'), t('Items'), t('Operations'));
  //m2osw}
  $rows = array();
  while ($category = db_fetch_object($result)) {
    //{m2osw
    $rows[] = array($category->cid, l($category->title, "aggregator/categories/$category->cid"), format_plural($category->items, '1 item', '@count items'), l(t('edit'), "admin/content/aggregator/edit/category/$category->cid"));
    //m2osw}
  }
  $output .= theme('table', $header, $rows);

  return $output;
}

/**
 * Form builder; Generate a form to add/edit feed sources.
 *
 * @ingroup forms
 * @see aggregator_form_feed_validate()
 * @see aggregator_form_feed_submit()
 */
//{m2osw
function aggregator_form_feed(&$form_state, $edit = array('refresh' => 86400, 'title' => '', 'url' => '', 'fid' => NULL)) {
//m2osw}
  $period = drupal_map_assoc(array(900, 1800, 3600, 7200, 10800, 21600, 32400, 43200, 64800, 86400, 172800, 259200, 604800, 1209600, 2419200), 'format_interval');

  if ($edit['refresh'] == '') {
    $edit['refresh'] = 3600;
  }

  $form['title'] = array('#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $edit['title'],
    '#maxlength' => 255,
    '#description' => t('The name of the feed (or the name of the website providing the feed).'),
    '#required' => TRUE,
  );
  $form['url'] = array('#type' => 'textfield',
    '#title' => t('URL'),
    '#default_value' => $edit['url'],
    '#maxlength' => 255,
    '#description' => t('The fully-qualified URL of the feed.'),
    '#required' => TRUE,
  );
  $form['refresh'] = array('#type' => 'select',
    '#title' => t('Update interval'),
    '#default_value' => $edit['refresh'],
    '#options' => $period,
    '#description' => t('The length of time between feed updates. (Requires a correctly configured <a href="@cron">cron maintenance task</a>.)', array('@cron' => url('admin/reports/status'))),
  );
  //{m2osw
  $form['show_block'] = array('#type' => 'checkbox',
    '#title' => t('Show as block'),
    '#default_value' => $edit['show_block'],
    '#description' => t('Whether you want to use this feed as a block.'),
  );
  $form['node_type'] = array(
    '#type' => 'select',
    '#title' => t('Node type'),
    '#default_value' => $edit['node_type'],
    '#options' => array_merge(array('d-2' => t('Do not create nodes for this feed'), 'd-1' => t('Default')), node_get_types('names')),
    '#description' => t('When you select the "Create nodes" flag in the <a href="/admin/content/aggregator/settings">Settings</a>, this type is used to create new nodes. "Default" can be set in the settings and will likely default to "Story" until set by you.'),
  );
  //m2osw}

  // Handling of categories:
  $options = array();
  $values = array();
  $categories = db_query('SELECT c.cid, c.title, f.fid FROM {aggregator_category} c LEFT JOIN {aggregator_category_feed} f ON c.cid = f.cid AND f.fid = %d ORDER BY title', $edit['fid']);
  while ($category = db_fetch_object($categories)) {
    $options[$category->cid] = check_plain($category->title);
    if ($category->fid) $values[] = $category->cid;
  }
  if ($options) {
    $form['category'] = array('#type' => 'checkboxes',
      //{m2osw
      '#title' => t('Categorize news items (Aggregator specific)'),
      //m2osw}
      '#default_value' => $values,
      '#options' => $options,
      '#description' => t('New feed items are automatically filed in the checked categories.'),
    );
  }

  //{m2osw
  // Handling of taxonomies:
  $taxonomies = variable_get('aggregator_taxonomies', array());
  foreach ($taxonomies as $vid) {
    $vocabulary = taxonomy_vocabulary_load($vid);
    $all_terms = taxonomy_get_tree($vid);
    $terms = array();
    foreach ($all_terms as $t) {
      $terms[$t->tid] = $t->name;
    }

    // get the terms for that vocabulary
    $default_terms = array();
    if ($edit['fid']) {
      $result = db_query('SELECT t.tid, t.name FROM {term_data} t, {aggregator_feed_terms} f WHERE t.tid = f.tid AND f.fid = %d AND t.vid = %d', $edit['fid'], $vid);
      while ($term = db_fetch_object($result)) {
        $default_terms[$term->tid] = $term->tid;
      }
    }

    $form['taxonomy']['taxonomy'.$vocabulary->vid] = array(
      '#type' => $vocabulary->multiple ? 'checkboxes' : 'radios',
      '#title' => $vocabulary->name,
      '#description' => $vocabulary->multiple ? t('Select all that apply') : t('Select one term'),
      '#required' => $vocabulary->required,
      '#options' => $terms,
      '#default_value' => $default_terms,
      '#weight' => $vocabulary->weight,
    );
  }
  //m2osw}

  $form['submit'] = array('#type' => 'submit', '#value' => t('Save'));

  if ($edit['fid']) {
    $form['delete'] = array('#type' => 'submit', '#value' => t('Delete'));
    $form['fid'] = array('#type' => 'hidden', '#value' => $edit['fid']);
  }

  return $form;
}

/**
 * Validate aggregator_form_feed form submissions.
 */
function aggregator_form_feed_validate($form, &$form_state) {
  if ($form_state['values']['op'] == t('Save')) {
    // Ensure URL is valid.
    if (!valid_url($form_state['values']['url'], TRUE)) {
      form_set_error('url', t('The URL %url is invalid. Please enter a fully-qualified URL, such as http://www.example.com/feed.xml.', array('%url' => $form_state['values']['url'])));
    }
    // Check for duplicate titles.
    if (isset($form_state['values']['fid'])) {
      $result = db_query("SELECT title, url FROM {aggregator_feed} WHERE (title = '%s' OR url = '%s') AND fid <> %d", $form_state['values']['title'], $form_state['values']['url'], $form_state['values']['fid']);
    }
    else {
      $result = db_query("SELECT title, url FROM {aggregator_feed} WHERE title = '%s' OR url = '%s'", $form_state['values']['title'], $form_state['values']['url']);
    }
    while ($feed = db_fetch_object($result)) {
      if (strcasecmp($feed->title, $form_state['values']['title']) == 0) {
        form_set_error('title', t('A feed named %feed already exists. Please enter a unique title.', array('%feed' => $form_state['values']['title'])));
      }
      if (strcasecmp($feed->url, $form_state['values']['url']) == 0) {
        form_set_error('url', t('A feed with this URL %url already exists. Please enter a unique URL.', array('%url' => $form_state['values']['url'])));
      }
    }
  }
}

/**
 * Process aggregator_form_feed form submissions.
 *
 * @todo Add delete confirmation dialog.
 */
function aggregator_form_feed_submit($form, &$form_state) {
  //{m2osw
  variable_set('aggregator_done_with_item_nodes', TRUE);
  //m2osw}
  if ($form_state['values']['op'] == t('Delete')) {
    $title = $form_state['values']['title'];
    // Unset the title:
    unset($form_state['values']['title']);
  }
  aggregator_save_feed($form_state['values']);
  if (isset($form_state['values']['fid'])) {
    if (isset($form_state['values']['title'])) {
      drupal_set_message(t('The feed %feed has been updated.', array('%feed' => $form_state['values']['title'])));
      if (arg(0) == 'admin') {
        $form_state['redirect'] = 'admin/content/aggregator/';
        return;
      }
      else {
        $form_state['redirect'] = 'aggregator/sources/'. $form_state['values']['fid'];
        return;
      }
    }
    else {
      watchdog('aggregator', 'Feed %feed deleted.', array('%feed' => $title));
      drupal_set_message(t('The feed %feed has been deleted.', array('%feed' => $title)));
      if (arg(0) == 'admin') {
        $form_state['redirect'] = 'admin/content/aggregator/';
        return;
      }
      else {
        $form_state['redirect'] = 'aggregator/sources/';
        return;
      }
    }
  }
  else {
    watchdog('aggregator', 'Feed %feed added.', array('%feed' => $form_state['values']['title']), WATCHDOG_NOTICE, l(t('view'), 'admin/content/aggregator'));
    drupal_set_message(t('The feed %feed has been added.', array('%feed' => $form_state['values']['title'])));
  }
}

function aggregator_admin_remove_feed($form_state, $feed) {
  return confirm_form(
    array(
      'feed' => array(
        '#type' => 'value',
        '#value' => $feed,
      ),
    ),
    t('Are you sure you want to remove all items from the feed %feed?', array('%feed' => $feed['title'])),
    'admin/content/aggregator',
    t('This action cannot be undone.'),
    t('Remove items'),
    t('Cancel')
  );
}

/**
 * Remove all items from a feed and redirect to the overview page.
 *
 * @param $feed
 *   An associative array describing the feed to be cleared.
 */
function aggregator_admin_remove_feed_submit($form, &$form_state) {
  aggregator_remove($form_state['values']['feed']);
  $form_state['redirect'] = 'admin/content/aggregator';
}

/**
 * Menu callback; refreshes a feed, then redirects to the overview page.
 *
 * @param $feed
 *   An associative array describing the feed to be refreshed.
 */
function aggregator_admin_refresh_feed($feed) {
  aggregator_refresh($feed);
  drupal_goto('admin/content/aggregator');
}

/**
 * Form builder; Configure the aggregator system.
 *
 * @ingroup forms
 * @see system_settings_form()
 */
function aggregator_admin_settings() {
  $items = array(0 => t('none')) + drupal_map_assoc(array(3, 5, 10, 15, 20, 25), '_aggregator_items');
  $period = drupal_map_assoc(array(3600, 10800, 21600, 32400, 43200, 86400, 172800, 259200, 604800, 1209600, 2419200, 4838400, 9676800), 'format_interval');
  //{m2osw
  $period[0] = 'Never';
  //m2osw}

  //{m2osw
  // this should be done in a submit function, but it won't matter much
  // (we only lose one SQL query in the next CRON)
  variable_set('aggregator_done_with_item_nodes', TRUE);
  //m2osw}

  $form['aggregator_allowed_html_tags'] = array(
    '#type' => 'textfield', '#title' => t('Allowed HTML tags'), '#size' => 80, '#maxlength' => 255,
    '#default_value' => variable_get('aggregator_allowed_html_tags', '<a> <b> <br> <dd> <dl> <dt> <em> <i> <li> <ol> <p> <strong> <u> <ul>'),
    '#description' => t('A space-separated list of HTML tags allowed in the content of feed items. (Tags in this list are not removed by Drupal.)')
  );

  $form['aggregator_summary_items'] = array(
    '#type' => 'select', '#title' => t('Items shown in sources and categories pages') ,
    '#default_value' => variable_get('aggregator_summary_items', 3), '#options' => $items,
    '#description' => t('Number of feed items displayed in feed and category summary pages.')
  );

  $form['aggregator_clear'] = array(
    '#type' => 'select', '#title' => t('Discard items older than'),
    '#default_value' => variable_get('aggregator_clear', 9676800), '#options' => $period,
    '#description' => t('The length of time to retain feed items before discarding. (Requires a correctly configured <a href="@cron">cron maintenance task</a>.)', array('@cron' => url('admin/reports/status')))
  );

  $form['aggregator_category_selector'] = array(
    '#type' => 'radios', '#title' => t('Category selection type'), '#default_value' => variable_get('aggregator_category_selector', 'checkboxes'),
    '#options' => array('checkboxes' => t('checkboxes'), 'select' => t('multiple selector')),
    '#description' => t('The type of category selection widget displayed on categorization pages. (For a small number of categories, checkboxes are easier to use, while a multiple selector work well with large numbers of categories.)')
  );

  //{m2osw
  $form['aggregator_create_nodes'] = array(
    '#type' => 'checkbox',
    '#title' => t('Create nodes'),
    '#default_value' => variable_get('aggregator_create_nodes', FALSE),
    '#description' => t('When selected, the aggregator also creates a node for each item it creates.'),
  );

  $form['aggregator_default_node_type'] = array(
    '#type' => 'select',
    '#title' => t('Default node type'),
    '#default_value' => variable_get('aggregator_default_node_type', NULL),
    '#options' => array_merge(array('d-2' => 'By default, do not create nodes for feeds'), node_get_types('names')),
    '#description' => t('Feeds marked to use the default node type will use this type. If you select "By default, do not create nodes for feeds" then feeds that do not specify a node type will not get nodes for their items until you go make a selection.'),
  );

  $form['aggregator_node_author'] = array(
    '#type' => 'textfield',
    '#title' => t('Mark nodes as authored by'),
    '#maxlength' => 60,
    '#autocomplete_path' => 'user/autocomplete',
    '#default_value' => variable_get('aggregator_node_author', NULL),
    '#description' => t('Leave blank for %anonymous.', array('%anonymous' => variable_get('anonymous', t('Anonymous')))),
  );

  $formats = filter_formats();
  $options = array(0 => 'Use the system default format');
  foreach ($formats as $f) {
    $options[$f->format] = $f->name;
  }
  $form['aggregator_node_format'] = array(
    '#type' => 'select',
    '#title' => t('Nodes format'),
    '#default_value' => variable_get('aggregator_node_format', 0),
    '#options' => $options,
    '#description' => t('All the nodes created by the aggregator will use this format. The safest is generally to use the system Default format. Remember that you do not have direct control over the data published in public feeds.'),
  );

  $form['aggregator_check_item_count'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of items to check in CRON'),
    '#default_value' => variable_get('aggregator_check_item_count', 0),
    '#description' => t('Each time CRON runs, the aggregator will check your existing items to know whether or not to add them as nodes. To use the default, enter 0. The allowed range is 250 to 10000.'),
  );
  //m2osw}

  return system_settings_form($form);
}

/**
 * Form builder; Generate a form to add/edit/delete aggregator categories.
 *
 * @ingroup forms
 * @see aggregator_form_category_validate()
 * @see aggregator_form_category_submit()
 */
function aggregator_form_category(&$form_state, $edit = array('title' => '', 'description' => '', 'cid' => NULL)) {
  $form['title'] = array('#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $edit['title'],
    '#maxlength' => 64,
    '#required' => TRUE,
  );
  $form['description'] = array('#type' => 'textarea',
    '#title' => t('Description'),
    '#default_value' => $edit['description'],
  );
  //{m2osw
  $form['show_block'] = array('#type' => 'checkbox',
    '#title' => t('Show as block'),
    '#default_value' => $edit['show_block'],
    '#description' => t('Whether you want to use this category as a block.'),
  );
  //m2osw}
  $form['submit'] = array('#type' => 'submit', '#value' => t('Save'));

  if ($edit['cid']) {
    $form['delete'] = array('#type' => 'submit', '#value' => t('Delete'));
    $form['cid'] = array('#type' => 'hidden', '#value' => $edit['cid']);
  }

  return $form;
}

/**
 * Validate aggregator_form_feed form submissions.
 */
function aggregator_form_category_validate($form, &$form_state) {
  if ($form_state['values']['op'] == t('Save')) {
    // Check for duplicate titles
    if (isset($form_state['values']['cid'])) {
      $category = db_fetch_object(db_query("SELECT cid FROM {aggregator_category} WHERE title = '%s' AND cid <> %d", $form_state['values']['title'], $form_state['values']['cid']));
    }
    else {
      $category = db_fetch_object(db_query("SELECT cid FROM {aggregator_category} WHERE title = '%s'", $form_state['values']['title']));
    }
    if ($category) {
      form_set_error('title', t('A category named %category already exists. Please enter a unique title.', array('%category' => $form_state['values']['title'])));
    }
  }
}

/**
 * Process aggregator_form_category form submissions.
 *
 * @todo Add delete confirmation dialog.
 */
function aggregator_form_category_submit($form, &$form_state) {
  if ($form_state['values']['op'] == t('Delete')) {
    $title = $form_state['values']['title'];
    // Unset the title:
    unset($form_state['values']['title']);
  }
  aggregator_save_category($form_state['values']);
  if (isset($form_state['values']['cid'])) {
    if (isset($form_state['values']['title'])) {
      drupal_set_message(t('The category %category has been updated.', array('%category' => $form_state['values']['title'])));
      if (arg(0) == 'admin') {
        $form_state['redirect'] = 'admin/content/aggregator/';
        return;
      }
      else {
        $form_state['redirect'] = 'aggregator/categories/'. $form_state['values']['cid'];
        return;
      }
    }
    else {
      watchdog('aggregator', 'Category %category deleted.', array('%category' => $title));
      drupal_set_message(t('The category %category has been deleted.', array('%category' => $title)));
      if (arg(0) == 'admin') {
        $form_state['redirect'] = 'admin/content/aggregator/';
        return;
      }
      else {
        $form_state['redirect'] = 'aggregator/categories/';
        return;
      }
    }
  }
  else {
    watchdog('aggregator', 'Category %category added.', array('%category' => $form_state['values']['title']), WATCHDOG_NOTICE, l(t('view'), 'admin/content/aggregator'));
    drupal_set_message(t('The category %category has been added.', array('%category' => $form_state['values']['title'])));
  }
}

//{m2osw
/**
 * This function generates the necessary form items for the user
 * to select whether this taxonomy is to be used with Aggregators.
 */
function _aggregator_taxonomy_settings(&$form) {
  $vid = isset($form['vid']['#value']) ? $form['vid']['#value'] : 0;

  $taxonomies = variable_get('aggregator_taxonomies', array());
  $form['content_types']['aggregator_taxonomy'] = array(
    '#type' => 'checkbox',
    '#title' => t('Aggregator category'),
    '#default_value' => isset($taxonomies[$vid]) ? $taxonomies[$vid] : 0,
    '#description' => t('When checked, the taxonomy terms appear on the settings page of the aggregator feeds. Thus letting you categories the feed items that are transformed into nodes.'),
  );

  $form['#submit'][] = '_aggregator_form_vocabulary_submit';
}
//m2osw}
